/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : WeightedRandomPickerSet.cc                               */
/*                                                                       */
/*       Date : October 13, 1997                                         */
/* Updated at : March   14, 1999                                         */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/


#include <iostream.h>
#include <math.h>
#include <stdlib.h>
#include "WeightedRandomPickerSet.h"
#include "tSet.h"
#include "gVector.h"

/*************************************************************************/
/*************************************************************************/
template<class T>
WeightedRandomPickerSet<T>::WeightedRandomPickerSet() : tSet<T>() { }

/*************************************************************************/
/*************************************************************************/
template<class T>
WeightedRandomPickerSet<T>::WeightedRandomPickerSet(int size) : tSet<T>(size)
{
  gVector<float> a(size);
  probs = a;
}

/*************************************************************************/
/*************************************************************************/
template<class T>
WeightedRandomPickerSet<T>::WeightedRandomPickerSet(const WeightedRandomPickerSet<T>& RHS) 
{  
   copy(RHS);
   probs = RHS.probs;

}// end of COPY CONSTRUCTOR

/*************************************************************************/
/*************************************************************************/
template<class T>
WeightedRandomPickerSet<T>
WeightedRandomPickerSet<T>::operator=(const WeightedRandomPickerSet<T>& RHS) 
{  
   copy(RHS);
   probs = RHS.probs;

   return (*this);

}// end of operator=()

/*************************************************************************/
/*************************************************************************/
template<class T>
WeightedRandomPickerSet<T>
WeightedRandomPickerSet<T>::operator+(const WeightedRandomPickerSet<T>& RHS) 
{  
   WeightedRandomPickerSet<T> ret = RHS;
   T ele; 

   for(int i=0;i<index;i++) 
       if( getElementAtSafe(i,ele) ) ret.addElement(ele,*probs.getElement(i));

   return ret;

}// end of operator+()

/*************************************************************************/
/*************************************************************************/
template<class T>
WeightedRandomPickerSet<T>::~WeightedRandomPickerSet()
{ }// end of Destructor

/*************************************************************************/
/*************************************************************************/
template<class T>
void WeightedRandomPickerSet<T>::printMe(ostream& os) const
{
   tSet<T>::printMe(os);
   int k = 0;
   float *ele;

   os << " { ";

   for(int i=0;i<index;i++) 
   {
      ele = probs.getElement(i);
      if(ele!=NULL)
      {

	if(i<(index-1)) os << *ele << " , ";
        else  os << *ele << " } " << endl;

        k++;
        if((k%5)==0) os << endl;

      }// end of if

   }// end of for

}// end of printMe()

/*************************************************************************/
/*************************************************************************/
template<class T>
void WeightedRandomPickerSet<T>::setProbabilityOfElementAt(unsigned indx, float val )
{  
   if(indx>=0 && indx < index) probs.replaceElementAt(indx,val);

}// end of setProbabilityOfElementAt()

/*************************************************************************/
/*************************************************************************/
template<class T>
float WeightedRandomPickerSet<T>::getProbabilityOfElementAt(unsigned indx)
{  
   float ret = 0.0;
   if(indx>=0 && indx < index) ret = *probs.getElement(indx);

   return ret;

}// end of setProbabilityOfElementAt()

/*************************************************************************/
/** Picks the index element randomly according to its preference.  */
/*************************************************************************/
template<class T>
WeightedRandomPickerSet<T>::findIndex(int& retValue) const
{
   int i, indx = 0;
   int ret=0;

   if(index>0)
   {
      float tot = 0;

      for(i=0;i<index;i++) tot += *probs.getElement(i);

      float pick = RandomGenerator::generateFloat(0,tot);

      tot = 0.0; i=0;

      while((tot+*probs.getElement(i))<pick && i<index)
      {
	 tot += *probs.getElement(i);
	 i++;

      }// end of while

      retValue = i % index;

      ret = 1;

   }// end of if

   return ret;

}// end of randomPick()

/*************************************************************************/
/** Picks the element randomly according to its preference.  */
/*************************************************************************/
template<class T>
T *WeightedRandomPickerSet<T>::randomPick() const
{
   int i ;
   T *ret = NULL;

   if(findIndex(i)) ret = getElementAt(i);

   return ret;

}// end of randomPick()

/*************************************************************************/
/** Picks the element randomly according to its preference.  */
/*************************************************************************/
template<class T>
WeightedRandomPickerSet<T>::randomPickSafe(T& retValue) const
{
   int i ;
   int ret=findIndex(i);

   if(ret) getElementAtSafe(i,retValue);

   return ret;

}// end of randomPickSafe()

/*************************************************************************/
/**
    Add value to the set, update its probability.
**/
/*************************************************************************/
template<class T>
WeightedRandomPickerSet<T>::addElement(T val, float _prob)
{
   int ret = addElement(val);
   if(ret) probs.replaceElementAt(index-1,_prob);
   return ret;
}// end of addElement()

/*************************************************************************/
/**
**/
/*************************************************************************/
template<class T>
WeightedRandomPickerSet<T>::addElement(T val)
{
   int indx = index;
   int ret = tSet<T>::addElement(val);
   
   if(ret==1) probs.addElement(1.0);

   return ret;

}// end of add

/*************************************************************************/
/*************************************************************************/
template<class T>
WeightedRandomPickerSet<T>::removeElement(T val)
{
   int indx = getIndexOfElement(val);
   int ret = 0;

   if(indx>-1)  ret = removeElementAt(indx); 
   
   return ret;

}// end of removeElement()

/*************************************************************************/
/*************************************************************************/
template<class T>
WeightedRandomPickerSet<T>::removeElementAt(unsigned indx)
{
   int pindx = index;
   int ret = tSet<T>::removeElementAt(indx);

   if(ret) probs.removeElementAt(indx);

   return ret;

}// end of removeElementAt()

